@@ -208,11 +208,19 @@ class ChunkedCrossAttention(nn.Module):
 
         x = F.pad(x, (0, 0, -causal_padding, causal_padding), value = 0.)
 
+        # remove sequence which is ahead of the neighbors retrieved (during inference)
+
+        seq_index = (n // chunk_size) * chunk_size
+        x, x_remainder = x[:, :seq_index], x[:, seq_index:]
+
+        seq_remain_len = x_remainder.shape[-2]
+
         # take care of rotary positional embedding
         # make sure queries positions are properly shifted to the future
 
         q_pos_emb, k_pos_emb = pos_emb
         q_pos_emb = F.pad(q_pos_emb, (0, 0, -causal_padding, causal_padding), value = 0.)
+
         k_pos_emb = repeat(k_pos_emb, 'b h n d -> b h (r n) d', r = num_retrieved)
         pos_emb = (q_pos_emb, k_pos_emb)
 
@@ -234,7 +242,7 @@ class ChunkedCrossAttention(nn.Module):
 
         # pad back to original, with 0s at the beginning (which will be added to the residual and be fine)
 
-        out = F.pad(out, (0, 0, causal_padding, -causal_padding), value = 0.)
+        out = F.pad(out, (0, 0, causal_padding, -causal_padding + seq_remain_len), value = 0.)
         return out
 
 # encoder and decoder classes
@@ -320,21 +328,22 @@ class Decoder(nn.Module):
 
         self.norm_out = RMSNorm(dim) if final_norm else nn.Identity()
 
-    def forward(self, x, *, context_mask = None, retrieved):
-        device, seq_len, num_chunks, num_neighbors, chunk_size = x.device, x.shape[-2], *retrieved.shape[-4:-1]
-        seq_chunk_size = seq_len // num_chunks
-
+    def forward(self, x, *, context_mask = None, retrieved = None):
+        device, seq_len = x.device, x.shape[-2]
         self_attn_pos_emb = self.rotary_pos_emb(seq_len, device = device)
 
-        cross_attn_q_pos_emb = self.rotary_pos_emb(seq_chunk_size, device = device, offset = self.chunk_size - 1)  # need to add extra chunk size, since it will be shifted
-        cross_attn_k_pos_emb = self.rotary_pos_emb(chunk_size, device = device)
+        if exists(retrieved):
+            num_chunks, num_neighbors, chunk_size = retrieved.shape[-4:-1]
 
-        cross_attn_pos_emb = (cross_attn_q_pos_emb, cross_attn_k_pos_emb)
+            cross_attn_q_pos_emb = self.rotary_pos_emb(self.chunk_size, device = device, offset = self.chunk_size - 1)  # need to add extra chunk size, since it will be shifted
+            cross_attn_k_pos_emb = self.rotary_pos_emb(chunk_size, device = device)
+
+            cross_attn_pos_emb = (cross_attn_q_pos_emb, cross_attn_k_pos_emb)
 
         for attn, cross_attn, ff in self.layers:
             x = attn(x, pos_emb = self_attn_pos_emb) + x
 
-            if exists(cross_attn):
+            if exists(cross_attn) and exists(retrieved):
                 x = cross_attn(
                     x,
                     context = retrieved,
@@ -401,10 +410,32 @@ class RETRO(nn.Module):
 
         self.to_logits = nn.Linear(dec_dim, num_tokens)
 
+    def forward_without_retrieval(
+        self,
+        seq
+    ):
+        # embed sequence
+
+        embed = self.token_emb(seq)
+        embed = embed[:, :self.seq_len]
+
+        # get absolute positional embedding
+
+        pos_emb = self.pos_emb(torch.arange(embed.shape[1], device = embed.device))
+        pos_emb = rearrange(pos_emb, 'n d -> 1 n d')
+        embed = embed + pos_emb
+
+        embed = self.to_decoder_model_dim(embed)
+        embed = self.decoder(embed)
+
+        # project to logits
+
+        return self.to_logits(embed)
+
     def forward(
         self,
         seq,
-        retrieved,
+        retrieved = None,
         return_loss = False
     ):
         """
@@ -415,6 +446,9 @@ class RETRO(nn.Module):
         r - num retrieved neighbors
         """
 
+        if not exists(retrieved):
+            return self.forward_without_retrieval(seq)
+
         assert not (return_loss and not self.training), 'must be training if returning loss'
 
         # assume padding token id (usually 0.) is to be masked out
@@ -436,7 +470,13 @@ class RETRO(nn.Module):
         n, num_chunks, num_neighbors, chunk_size, retrieved_shape, device = seq.shape[-1], *retrieved.shape[-3:], retrieved.shape, seq.device
 
         assert chunk_size >= self.chunk_size, 'chunk size of retrieval input must be greater or equal to the designated chunk_size on RETRO initialization'
-        assert divisible_by(n, self.chunk_size), 'sequence length must be divisible by chunk size'
+
+        num_seq_chunks = n // self.chunk_size
+        assert num_chunks == num_seq_chunks, f'sequence requires {num_seq_chunks} retrieved chunks, but only {num_chunks} passed in'
+
+        # sequence index at which k-nearest neighbors have not been fetched yet after
+
+        seq_index = num_seq_chunks * self.chunk_size
 
         # embed both sequence and retrieved chunks
 
@@ -461,7 +501,7 @@ class RETRO(nn.Module):
         # encode
 
         retrieved = rearrange(retrieved, 'b k r n d -> (b k r) n d')
-        embed_as_context = repeat(embed, 'b (k n) d -> (b k r) n d', n = self.chunk_size, r = num_neighbors)
+        embed_as_context = repeat(embed[:, :seq_index], 'b (k n) d -> (b k r) n d', n = self.chunk_size, r = num_neighbors)
 
         retrieved = self.encoder(retrieved, mask = encoder_retrieved_mask, chunked_seq = embed_as_context)
         retrieved = rearrange(retrieved, '(b k r) n d -> b k r n d', k = num_chunks, r = num_neighbors)
